@@ -7,7 +7,7 @@ use tracing_subscriber::filter::LevelFilter;
 
 use pea2pea::{
     protocols::{Handshaking, Reading, Writing},
-    Connection, ConnectionSide, Node, NodeConfig, Pea2Pea,
+    Config, Connection, ConnectionSide, Node, Pea2Pea,
 };
 
 use std::{io, net::SocketAddr, time::Duration};
@@ -27,16 +27,16 @@ impl Handshaking for JoJoNode {
         // some handshakes are useful, others are menacing ゴゴゴゴ
         match !conn.side {
             ConnectionSide::Initiator => {
-                info!(parent: conn.node.span(), "Dio!");
+                info!(parent: self.node().span(), "Dio!");
                 sleep(Duration::from_secs(4)).await;
-                info!(parent: conn.node.span(), "I can't beat the shit out of you without getting closer.");
+                info!(parent: self.node().span(), "I can't beat the shit out of you without getting closer.");
                 sleep(Duration::from_secs(3)).await;
             }
             ConnectionSide::Responder => {
                 sleep(Duration::from_secs(1)).await;
-                warn!(parent: conn.node.span(), "Oh, you're approaching me? Instead of running away, you're coming right to me?");
+                warn!(parent: self.node().span(), "Oh, you're approaching me? Instead of running away, you're coming right to me?");
                 sleep(Duration::from_secs(6)).await;
-                warn!(parent: conn.node.span(), "Oh ho! Then come as close as you like.");
+                warn!(parent: self.node().span(), "Oh ho! Then come as close as you like.");
             }
         }
 
@@ -90,9 +90,14 @@ impl Reading for JoJoNode {
 }
 
 impl Writing for JoJoNode {
-    fn write_message(&self, _: SocketAddr, payload: &[u8], buffer: &mut [u8]) -> io::Result<usize> {
-        buffer[0] = payload[0];
-        let battle_cry = BattleCry::from(buffer[0]);
+    fn write_message<W: io::Write>(
+        &self,
+        _: SocketAddr,
+        payload: &[u8],
+        writer: &mut W,
+    ) -> io::Result<()> {
+        writer.write_all(&payload[..1])?;
+        let battle_cry = BattleCry::from(payload[0]);
 
         if battle_cry == BattleCry::Ora {
             info!(parent: self.node().span(), "{:?}!", battle_cry);
@@ -100,7 +105,7 @@ impl Writing for JoJoNode {
             warn!(parent: self.node().span(), "{:?}!", battle_cry);
         };
 
-        Ok(1)
+        Ok(())
     }
 }
 
@@ -108,14 +113,14 @@ impl Writing for JoJoNode {
 async fn main() {
     common::start_logger(LevelFilter::INFO);
 
-    let config = NodeConfig {
+    let config = Config {
         name: Some("Jotaro".into()),
         max_handshake_time_ms: 10_000,
         ..Default::default()
     };
     let jotaro = JoJoNode(Node::new(Some(config)).await.unwrap());
 
-    let config = NodeConfig {
+    let config = Config {
         name: Some("Dio".into()),
         max_handshake_time_ms: 10_000,
         ..Default::default()
