@@ -5,7 +5,7 @@ use rand::{distributions::Standard, rngs::SmallRng, Rng, SeedableRng};
 mod common;
 use pea2pea::{
     protocols::{Reading, Writing},
-    Node, NodeConfig, Pea2Pea,
+    Config, Node, Pea2Pea,
 };
 
 use std::{io, net::SocketAddr, time::Instant};
@@ -69,9 +69,8 @@ async fn run_bench_scenario(sender_count: usize) -> f64 {
     const NUM_MESSAGES: usize = 10_000;
     const MSG_SIZE: usize = 64 * 1024;
 
-    let config = NodeConfig {
-        conn_write_buffer_size: MSG_SIZE,
-        conn_outbound_queue_depth: NUM_MESSAGES,
+    let config = Config {
+        outbound_queue_depth: NUM_MESSAGES,
         ..Default::default()
     };
     let spammers = common::start_nodes(sender_count, Some(config)).await;
@@ -84,8 +83,8 @@ async fn run_bench_scenario(sender_count: usize) -> f64 {
         spammer.enable_writing();
     }
 
-    let config = NodeConfig {
-        conn_read_buffer_size: MSG_SIZE * 3,
+    let config = Config {
+        read_buffer_size: MSG_SIZE * 3,
         max_connections: sender_count as u16,
         ..Default::default()
     };
