@@ -9,7 +9,7 @@ use tracing::*;
 mod common;
 use pea2pea::{
     protocols::{Handshaking, Reading, Writing},
-    Connection, ConnectionSide, Node, NodeConfig, Pea2Pea,
+    Config, Connection, ConnectionSide, Node, Pea2Pea,
 };
 
 use std::{collections::HashMap, convert::TryInto, io, net::SocketAddr, sync::Arc};
@@ -65,18 +65,18 @@ impl Pea2Pea for SecureishNode {
 }
 
 macro_rules! read_handshake_message {
-    ($expected: path, $conn: expr) => {{
+    ($expected: path, $conn: expr, $node: expr) => {{
         let mut buf = [0u8; 9];
 
         $conn.reader().read_exact(&mut buf).await?;
         let msg = HandshakeMsg::deserialize(&buf)?;
 
         if let $expected(nonce) = msg {
-            debug!(parent: $conn.node.span(), "received {:?} from {}", msg, $conn.addr);
+            debug!(parent: $node.node().span(), "received {:?} from {}", msg, $conn.addr);
             nonce
         } else {
             error!(
-                parent: $conn.node.span(),
+                parent: $node.node().span(),
                 "received an invalid handshake message from {} (expected {}, got {:?})",
                 $conn.addr, stringify!($expected), msg,
             );
@@ -86,12 +86,12 @@ macro_rules! read_handshake_message {
 }
 
 macro_rules! send_handshake_message {
-    ($msg: expr, $conn: expr) => {
+    ($msg: expr, $conn: expr, $node: expr) => {
         $conn.writer()
             .write_all(&$msg.serialize())
             .await?;
 
-        debug!(parent: $conn.node.span(), "sent {:?} to {}", $msg, $conn.addr);
+        debug!(parent: $node.node().span(), "sent {:?} to {}", $msg, $conn.addr);
     }
 }
 
@@ -104,20 +104,20 @@ impl Handshaking for SecureishNode {
             ConnectionSide::Initiator => {
                 // send A
                 let own_nonce = 0;
-                send_handshake_message!(HandshakeMsg::A(own_nonce), conn);
+                send_handshake_message!(HandshakeMsg::A(own_nonce), conn, self);
 
                 // read B
-                let peer_nonce = read_handshake_message!(HandshakeMsg::B, conn);
+                let peer_nonce = read_handshake_message!(HandshakeMsg::B, conn, self);
 
                 NoncePair(own_nonce, peer_nonce)
             }
             ConnectionSide::Responder => {
                 // read A
-                let peer_nonce = read_handshake_message!(HandshakeMsg::A, conn);
+                let peer_nonce = read_handshake_message!(HandshakeMsg::A, conn, self);
 
                 // send B
                 let own_nonce = 1;
-                send_handshake_message!(HandshakeMsg::B(own_nonce), conn);
+                send_handshake_message!(HandshakeMsg::B(own_nonce), conn, self);
 
                 NoncePair(own_nonce, peer_nonce)
             }
@@ -134,7 +134,7 @@ impl Handshaking for SecureishNode {
 async fn handshake_example() {
     // tracing_subscriber::fmt::init();
 
-    let initiator_config = NodeConfig {
+    let initiator_config = Config {
         name: Some("initiator".into()),
         ..Default::default()
     };
@@ -144,7 +144,7 @@ async fn handshake_example() {
         handshakes: Default::default(),
     };
 
-    let responder_config = NodeConfig {
+    let responder_config = Config {
         name: Some("responder".into()),
         ..Default::default()
     };
@@ -177,7 +177,7 @@ async fn handshake_example() {
 
 #[tokio::test]
 async fn no_handshake_no_messaging() {
-    let initiator_config = NodeConfig {
+    let initiator_config = Config {
         name: Some("initiator".into()),
         ..Default::default()
     };
@@ -187,7 +187,7 @@ async fn no_handshake_no_messaging() {
         handshakes: Default::default(),
     };
 
-    let responder_config = NodeConfig {
+    let responder_config = Config {
         name: Some("responder".into()),
         ..Default::default()
     };
@@ -241,7 +241,7 @@ async fn hung_handshake_fails() {
         }
     }
 
-    let config = NodeConfig {
+    let config = Config {
         max_handshake_time_ms: 10,
         ..Default::default()
     };
@@ -291,7 +291,7 @@ async fn timeout_when_spammed_with_connections() {
         }
     }
 
-    let config = NodeConfig {
+    let config = Config {
         max_handshake_time_ms: TIMEOUT_SECS * 1_000,
         max_connections: NUM_ATTEMPTS,
         ..Default::default()
