@@ -6,7 +6,7 @@ use tracing::*;
 mod common;
 use pea2pea::{
     protocols::{Reading, Writing},
-    Node, NodeConfig, Pea2Pea,
+    Config, Node, Pea2Pea,
 };
 use TestMessage::*;
 
@@ -72,10 +72,14 @@ impl Reading for EchoNode {
 }
 
 impl Writing for EchoNode {
-    fn write_message(&self, _: SocketAddr, payload: &[u8], buffer: &mut [u8]) -> io::Result<usize> {
-        buffer[..4].copy_from_slice(&(payload.len() as u32).to_le_bytes());
-        buffer[4..][..payload.len()].copy_from_slice(payload);
-        Ok(4 + payload.len())
+    fn write_message<W: io::Write>(
+        &self,
+        _: SocketAddr,
+        payload: &[u8],
+        writer: &mut W,
+    ) -> io::Result<()> {
+        writer.write_all(&(payload.len() as u32).to_le_bytes())?;
+        writer.write_all(payload)
     }
 }
 
@@ -87,7 +91,7 @@ async fn messaging_example() {
     shouter.enable_reading();
     shouter.enable_writing();
 
-    let picky_echo_config = NodeConfig {
+    let picky_echo_config = Config {
         name: Some("picky_echo".into()),
         ..Default::default()
     };
@@ -155,9 +159,9 @@ async fn drop_connection_on_oversized_message() {
     let writer = common::MessagingNode::new("writer").await;
     writer.enable_writing();
 
-    let config = NodeConfig {
+    let config = Config {
         name: Some("reader".into()),
-        conn_read_buffer_size: MSG_SIZE_LIMIT,
+        read_buffer_size: MSG_SIZE_LIMIT,
         ..Default::default()
     };
     let reader = common::MessagingNode(Node::new(Some(config)).await.unwrap());
@@ -168,12 +172,23 @@ async fn drop_connection_on_oversized_message() {
 
     wait_until!(1, reader.node().num_connected() == 1);
 
-    // when prefixed with length, it'll exceed MSG_SIZE_LIMIT, i.e. the read buffer size of the reader
-    let oversized_payload = vec![0u8; MSG_SIZE_LIMIT];
+    // when prefixed with length, it'll be equal to MSG_SIZE_LIMIT
+    let max_size_payload = vec![0u8; MSG_SIZE_LIMIT - 4];
 
     writer
         .node()
-        .send_direct_message(reader_addr, common::prefix_with_len(2, &oversized_payload))
+        .send_direct_message(reader_addr, max_size_payload.into())
+        .unwrap();
+
+    wait_until!(1, reader.node().stats().received() == (1, 10));
+
+    // this message exceed MSG_SIZE_LIMIT, i.e. the read buffer size of the reader,
+    // by a single byte
+    let oversized_payload = vec![0u8; MSG_SIZE_LIMIT - 3];
+
+    writer
+        .node()
+        .send_direct_message(reader_addr, oversized_payload.into())
         .unwrap();
 
     wait_until!(1, reader.node().num_connected() == 0);
@@ -214,7 +229,7 @@ async fn no_reading_no_delivery() {
     // writer sends a message
     writer
         .node()
-        .send_direct_message(reader_addr, common::prefix_with_len(4, &[0; 16]))
+        .send_direct_message(reader_addr, vec![0; 16].into())
         .unwrap();
 
     sleep(Duration::from_millis(10)).await;
@@ -238,7 +253,7 @@ async fn no_writing_no_delivery() {
     // writer tries to send a message
     assert!(writer
         .node()
-        .send_direct_message(reader_addr, common::prefix_with_len(4, &[0; 16]))
+        .send_direct_message(reader_addr, vec![0; 16].into())
         .is_err());
 
     sleep(Duration::from_millis(10)).await;
